
#include <extdll.h>			// always

#include <api_info.h>		// api_info_t, etc
#include <sdk_util.h>		// REG_SVR_COMMAND, etc
#include <meta_api.h>		// Plugin_info, etc
#include "statsme.h"	// me
#include <ctype.h>

#define MAX_BUF_LEN	1024 //max. buffor for asked value
#define MAX_DATA_LEN 2048
#define MAX_LEVELS 6

// converts "abc" to bit sum
int ReadFlags(const char* c) {
	int flags = atoi( c );
	if ( !flags && *c )
		while (*c)	{ 
			if ( isalpha(*c) )	flags |= ( 1 << ( tolower( *c ) -'a' ) );
			++c;
		}
	return flags;
}

// checks if string represents the number (null string also is a value!)
bool isDigit(const char* value) {
	while (value && *value){
		if ((*value>'9'||*value<'0')&&*value!='.'&&*value!='-')
			return false;
		++value;
	}
	return true;
}

// finds internal var and puts value to it
void setVar(const char* name, const char* value)
{
	myvar_t*s;

	for(s = sm_var;s;s=s->next) {
		if (!strcmp(s->name,name))
			break;
	}

	if ( !s ) {
		myvar_t*newo=new myvar_t;
		strcpy(newo->name,name);
		newo->next = sm_var;
		sm_var = newo;
		s = newo;
	}

	strcpy(s->string,value);
}

// returns value from internal vars
char* getVar(const char* name)
{
	myvar_t*s;

	for(s = sm_var;s;s=s->next) {
		if (!strcmp(s->name,name))
			break;
	}

	if ( !s ) {
		myvar_t*newo=new myvar_t;
		strcpy(newo->name,name);
		newo->string[0] = '0';
		newo->string[1] = 0;
		newo->next = sm_var;
		sm_var = newo;
		s = newo;
	}

	return s->string;
}

// do operation on values and returns result
const char* doVar(const char* one, int op, const char* two)
{
	static char value[MAX_BUF_LEN];
	if (isDigit( one )) {
		float var = 0;
		switch(op) {		
		case '+':
			var = atof(one) + atof(two);
			break;
		case '-':
			var = atof(one) - atof(two);
			break;
		case '*':
			var = atof(one) * atof(two);
			break;
		case '/':
			var = atof(one) / atof(two);
			break;
		}
		sprintf(value,  ( var - (int)var ) ?    "%.2f"  : "%.0f",var );
	}
	else if ( op == '+' ) {
		strcpy(value,one);
		strcat(value,two);
	}
	else {
		value[0]=0;
	}

	return value;
}

// do operation on internal var and updates its value
void makeVar(const char* name, int op, const char* value){

	static char newvalue[MAX_BUF_LEN];

	if (isDigit( value )) {
		float var = atof( getVar(name) );
		switch(op) {		
		case '=':
			var = atof(value);
			break;
		case '+':
			var += atof(value);
			break;
		case '-':
			var -= atof(value);
			break;
		case '*':
			var *= atof(value);
			break;
		case '/':
			var /= atof(value);
			break;
		}
		sprintf(newvalue, (var-(int)var) ? "%.2f" : "%.0f" ,var );
		setVar( name ,  newvalue  );
	}
	else if ( op == '=' ) {
		setVar( name , value );
	}
}

// find out what MOD we are running
const char* GET_MOD_NAME(int* id) {
	
	char game_dir[256], *game_name = NULL;
	int i = 0;

	GET_GAME_DIR(game_dir); // get dir

	while ( game_dir[i] ) // catch word after last '/'
		if (game_dir[i++] == '/')
			game_name = &game_dir[i];

	static struct mod_info_t {
		const char* name;
		int id;
	} mods[] = {
		{"cstrike",MOD_CSTRIKE},
		{"dod",MOD_DOD},
		{"tfc",MOD_TFC},
		{"valve",0}
	};

	for (i = 0; mods[i].id; ++i) // and compare with our mod database
		if ( game_name && !strcmpi( game_name, mods[i].name ) )	{
			*id = mods[i].id;
			break;
		}

	return mods[i].name; // in case of failure the name will be "valve"
}

char* UTIL_VarArgs( char *fmt, ... ){
	va_list	argptr;
	static char	string[2048];
	va_start (argptr, fmt);
	vsprintf (string, fmt,argptr);
	va_end (argptr);
	return string;
}

void UTIL_LogPrintf( char *fmt, ... ){
	va_list	argptr;
	static char	string[2048];
	va_start ( argptr, fmt );
	vsprintf ( string, fmt, argptr );
	va_end   ( argptr );
	ALERT( at_logged, "%s", string );
}

#define MAX_MOTD_CHUNK	  60
#define MAX_MOTD_LENGTH   1535

void UTIL_ShowMOTD( edict_t *client , char *pMotd, const char *szName)
{

	if ( gmsgServerName == 0) {
		LOG_MESSAGE(PLID, "ERROR: (servername) no type for message.");
		return;
	}

	if ( gmsgMOTD == 0) {
		LOG_MESSAGE(PLID, "ERROR: (motd) no type for message.");
		return;
	}

	MESSAGE_BEGIN( MSG_ONE, gmsgServerName, NULL, client );
		WRITE_STRING(szName);
	MESSAGE_END();

	static char	chunk[MAX_MOTD_CHUNK+1];
	char *aList = pMotd;
	int	ch_count = 0;

	while ( pMotd && *pMotd && ch_count < MAX_MOTD_LENGTH )
	{
		if ( strlen( pMotd ) < MAX_MOTD_CHUNK ) {
			strcpy( chunk, pMotd );
		}
		else {
			strncpy( chunk, pMotd, MAX_MOTD_CHUNK );
			chunk[MAX_MOTD_CHUNK] = 0;
		}

		ch_count += strlen( chunk );

		if ( ch_count < MAX_MOTD_LENGTH ) {
			pMotd = aList + ch_count; 
		}
		else {
			*pMotd = 0;
		}

		MESSAGE_BEGIN( MSG_ONE, gmsgMOTD, NULL, client );
			WRITE_BYTE( *pMotd ? FALSE : TRUE );
			WRITE_STRING( chunk );
		MESSAGE_END();
	}

	MESSAGE_BEGIN( MSG_ONE, gmsgServerName, NULL, client );
		WRITE_STRING(CVAR_GET_STRING("hostname"));
	MESSAGE_END();
}

unsigned short FixedUnsigned16( float value, float scale ){
	int output;
	output = value * scale;
	if ( output < 0 )
		output = 0;
	if ( output > 0xFFFF )
		output = 0xFFFF;
	return (unsigned short)output;
}

short FixedSigned16( float value, float scale ){
	int output;
	output = value * scale;
	if ( output > 32767 )
		output = 32767;
	if ( output < -32768 )
		output = -32768;
	return (short)output;
}

char* UTIL_SplitTextMessage(char *pMessage)
{
	static char bMessage[256];
	int current = 0;		//Source position
	int len = 0;			//Over all length
	int line_len = 0;		//Line length
	int new_len = 0;		//Length from last space
	int last_space = -1;	//Place of last place

	//Go throw tekst and move to new line if needed
	while ( pMessage[len] && len < 254) {
		if (pMessage[len] == ' ' && len < 70) {
			last_space = len;	
			new_len = 0;
		}
		if (pMessage[len] == '\n') {
			line_len = 1; 
			last_space = -1;
		}

		bMessage[len++] = pMessage[current++];
		line_len++;	
		new_len++;

		if (line_len > 69) {
			if (last_space == -1) {
				bMessage[len++] = '\n';
				line_len = 0;
			}
			else {
				bMessage[last_space] = '\n';
				line_len = new_len;
			}
			last_space = -1;
		}
	}

	bMessage[len] = 0;
	return bMessage;
}

void UTIL_HudMessage(const hudtextparms_t &textparms, char *pMessage, edict_t *pEntity)
{	
	if (pEntity){ // to specified player
		MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity );
	}
	else { // to the public
		MESSAGE_BEGIN(MSG_BROADCAST, SVC_TEMPENTITY, NULL);
	}

	WRITE_BYTE(29); 
	WRITE_BYTE(textparms.channel & 0xFF); 
	WRITE_SHORT(FixedSigned16(textparms.x, (1<<13) ));
	WRITE_SHORT(FixedSigned16(textparms.y, (1<<13) ));
	WRITE_BYTE(textparms.effect);
	WRITE_BYTE(textparms.r1);
	WRITE_BYTE(textparms.g1);
	WRITE_BYTE(textparms.b1);
	WRITE_BYTE(0);
	WRITE_BYTE(255);
	WRITE_BYTE(255);
	WRITE_BYTE(250);
	WRITE_BYTE(0);
	WRITE_SHORT(FixedUnsigned16(textparms.fadeinTime, (1<<8) ));
	WRITE_SHORT(FixedUnsigned16(textparms.fadeoutTime, (1<<8) ));
	WRITE_SHORT(FixedUnsigned16(textparms.holdTime, (1<<8) ));

	if ( textparms.effect == 2 )
		WRITE_SHORT(FixedUnsigned16(textparms.fxTime, (1<<8) ) );
	
	WRITE_STRING(pMessage);
	MESSAGE_END();
}

// used to check if argument specifing the player
bool CallsMe(player_t* pPlayer, const char* arg){
	if ( !pPlayer || !pPlayer->valid || !pPlayer->ingame || !pPlayer->pEdict )
		return false;
	if ( arg[0]=='*' && !arg[1] )
		return true;
	if ( arg[0]=='#' && GETPLAYERUSERID( pPlayer->pEdict ) == atoi(arg+1) )
		return true;
	if ( !strcmp( pPlayer->team_name , arg ) )
		return true;
	return false;
}

// converts integer to string representation (range from 1 to 9999)
void UTIL_IntToString(int value, char *output)
{
	static const char *words[] = {"zero","one ","two ","three ","four ",
		"five ", "six ","seven ","eight ","nine ","ten ", //10
		"eleven ","twelve ","thirteen ","fourteen ","fifteen ",
		"sixteen ","seventeen ","eighteen ","nineteen ", //19
		"twenty ","thirty ","fourty ", "fifty ","sixty ",
		"seventy ","eighty ","ninety ",//27
		"hundred ","thousand "};
	*output = 0;
	int tho = value / 1000;
	if (tho){
		strcat(output, words[ tho ] );	
		strcat(output, words[29] );	
		value =  value % 1000;
	}
	int hun = value / 100;
	if (hun) {
		strcat(output, words[ hun ] );	
		strcat(output, words[28] );	
		value =  value % 100;
	}
	int ten = value / 10;
	int	unit = value % 10;
	if ( ten )	strcat(output, words[ ( ten > 1 ) ? ( ten + 18 ) : ( unit + 10 ) ] );
	if ( ten != 1 && ( unit || (!value && !hun && !tho) ) ) strcat(output, words[ unit ] );
}



player_t * GetBestPlayer(int option, const char* team)
{
	player_t *pBest = NULL;

	int value1 = 0, value2 = 0, check1, check2;

	for (int a = 1; a <= gpGlobals->maxClients ; ++a)
	{
		player_t *pPlayer = GET_PLAYER_POINTER_I(a);

		if (!pPlayer->ingame || (team ? strcmpi(pPlayer->team_name,team) : false )) continue;

		if (option > 3)	{
			switch (option){
			case 4: //best killer
				check1 = pPlayer->gamestats.kills;
				check2 = pPlayer->gamestats.hs;
				break;
			case 5: //most disruptive
				check1 = pPlayer->gamestats.damage;
				check2 = pPlayer->gamestats.hits;
				break;
			case 6: //most accurate shots
				check1 = pPlayer->gamestats.hitgroup[HITGROUP_HEAD];
				check2 = pPlayer->gamestats.hitgroup[HITGROUP_STOMACH] + pPlayer->gamestats.hitgroup[HITGROUP_CHEST];
				break;
			}
		}
		else {
			switch (option){
			case 1: //best killer
				check1 = pPlayer->roundstats.kills;
				check2 = pPlayer->roundstats.hs;
				break;
			case 2: //most disruptive
				check1 = pPlayer->roundstats.damage;
				check2 = pPlayer->roundstats.hits;
				break;
			case 3: //most accurate shots
				check1 = pPlayer->roundstats.hitgroup[HITGROUP_HEAD];
				check2 = pPlayer->roundstats.hitgroup[HITGROUP_STOMACH] + pPlayer->roundstats.hitgroup[HITGROUP_CHEST];
				break;
			}
		}

		//go through all score and find out who is the best
		if ( value1 < check1 || ( value1 == check1 && value2 < check2) )
		{	
			value1 = check1;
			value2 = check2;
			pBest = pPlayer;
		}		
	}

	return pBest;
}

const char* GetPlayerValue(player_t *pPlayer, const char* src);

const char* GetStatsValue(stat_t* stats, const char* src)
{
	static char value[2][MAX_BUF_LEN];
	static int i = 0; i = i ? 0 : 1;

	if (!stats) {
		value[i][0]=0;
	}
	else if (!strcmp(src,"name")){
		if (stats->name)
			sprintf(value[i], "%s", stats->name);
		else
			value[i][0]=0;
	}
	else if (!strcmp(src,"kills")){
		sprintf(value[i], "%d", stats->kills);
	}
	else if (!strcmp(src,"deaths")){
		sprintf(value[i], "%d", stats->deaths);
	}
	else if (!strcmp(src,"damage")){
		sprintf(value[i], "%d", stats->damage);
	}
	else if (!strcmp(src,"shots")){
		sprintf(value[i], "%d", stats->shots);
	}
	else if (!strcmp(src,"hits")){
		sprintf(value[i], "%d", stats->hits);
	}
	else if (!strcmp(src,"tks")){
		sprintf(value[i], "%d", stats->tks);
	}
	else if (!strcmp(src,"hs")){
		sprintf(value[i], "%d", stats->hs);
	}
	else if (!strcmp(src,"clip")){
		sprintf(value[i], "%d", stats->clip);
	}
	else if (!strcmp(src,"ammo")){
		sprintf(value[i], "%d", stats->ammo);
	}
	else if (!strcmp(src,"score")){
		sprintf(value[i], "%d", stats->score);
	}
	else if (!strcmp(src,"head")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_HEAD] );
	}
	else if (!strcmp(src,"chest")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_CHEST] );
	}
	else if (!strcmp(src,"stomach")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_STOMACH] );
	}
	else if (!strcmp(src,"leftarm")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_LEFTARM] );
	}
	else if (!strcmp(src,"rightarm")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_RIGHTARM] );
	}
	else if (!strcmp(src,"leftleg")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_LEFTLEG] );
	}
	else if (!strcmp(src,"rightleg")){
		sprintf(value[i], "%d", stats->hitgroup[HITGROUP_RIGHTLEG] );
	}
	else if (!strcmp(src,"acc")){
		float acc = ACCURACY_FORMULA( stats->shots, stats->hits );
		sprintf(value[i], "%.2f", acc );
	}
	else if (!strcmp(src,"eff")){
		float eff = EFFEC_FORMULA( stats->kills, stats->deaths );
		sprintf(value[i], "%.2f", eff );
	}
	else if (!strncmp(src,"mkill.",6)){

		if (!score.best || (score.best_type != stats->switcher + 1) ){
			score.best = GetBestPlayer(stats->switcher + 1, stats->name );
			score.best_type = stats->switcher + 1;
		}

		return GetPlayerValue( score.best , &src[6] );
	}
	else if (!strncmp(src,"mdisr.",6)){

		if (!score.best || (score.best_type != stats->switcher + 2) ) {
			score.best = GetBestPlayer(stats->switcher + 2, stats->name );
			score.best_type = stats->switcher + 2;
		}

		return GetPlayerValue( score.best , &src[6] );
	}
	else if (!strncmp(src,"macc.",5)){

		if (!score.best ||  (score.best_type != stats->switcher + 3) ){
			score.best = GetBestPlayer(stats->switcher + 3, stats->name );
			score.best_type = stats->switcher + 3;
		}

		return GetPlayerValue( score.best , &src[5] );
	}
	else {
		value[i][0]=0;
	}

	return value[i];
}

const char* GetPlayerValue(player_t *pPlayer, const char* src)
{
	static char value[2][MAX_BUF_LEN];
	static int i = 0; i = i ? 0 : 1;

	if (!pPlayer) {
		value[i][0]=0;
	}
	else if (!strcmp(src,"name")) {// most frequantly used
		char* name = INFOKEY_VALUE(GET_INFOKEYBUFFER(pPlayer->pEdict),"name");
		int len = 0 , c;
		while (name && *name) {//don't allow to pass {, } and $ in player's nicks
			c = *name++;
			if (c == '$')
				c = 'S';
			else if (c == '{')
				c = '(';
			else if (c == '}')
				c = ')';
			value[i][len++] = c;
		}
		value[i][len] = 0;
	}
	else if (!strncmp(src,"rd.",3)){ // stats for current round
		return GetStatsValue( &pPlayer->roundstats, &src[3] );
	}
	else if (!strncmp(src,"gm.",3)){ // stats from all play on current map
		return GetStatsValue( &pPlayer->gamestats, &src[3] );
	}
	else if (!strncmp(src,"tm.rd.",6)){ // team score
		return GetStatsValue( &score.roundstats[pPlayer->team_index], &src[6] );
	}
	else if (!strncmp(src,"tm.gm.",6)){ // team score
		return GetStatsValue( &score.gamestats[pPlayer->team_index], &src[6] );
	}
	else if (!strncmp(src,"k.",2)){ // killer
		return GetPlayerValue( pPlayer->killer, &src[2] );
	}
	else if (!strncmp(src,"v.",2)){ // victim
		return GetPlayerValue( pPlayer->victim, &src[2] );
	}
	else if (!strncmp(src,"w.",2)){ // currently used weapon
		return GetStatsValue( pPlayer->weapon, &src[2] );
	}
	else if (src[0]=='w'&&(src[2]=='.'||src[3]=='.')){ // custom wepon stats
		int x = (src[2]=='.') ? (src[1]-'0') : ( ((src[1]-'0')*10) + (src[2]-'0') );    
		return GetStatsValue(&pPlayer->wpnstats[x],&src[(src[2]=='.')?3:4]);
	}
	else if (src[0]=='a'&&(src[2]=='.'||src[3]=='.')){ // specified stats from action with players
		int y = (src[2]=='.') ? (src[1]-'0') : ( ((src[1]-'0')*10) + (src[2] - '0') );    
		return GetStatsValue(&pPlayer->specstats[y],&src[(src[2]=='.')?3:4]);
	}
	else if (!strncmp(src,"st.",3)){ // custom stats - diffrent on each event
		return GetStatsValue( pPlayer->custom_stats, &src[3] );
	}
	else if (!strcmp(src,"victims")){
		return GetFightList(pPlayer, false);
	}
	else if (!strcmp(src,"attackers")){
		return GetFightList(pPlayer, true);
	}
	else if (!strcmp(src,"health")){
		sprintf(value[i], "%d", pPlayer->health );
	}
	else if (!strcmp(src,"armor")){
		sprintf(value[i], "%d", pPlayer->armor );
	}
	else if (!strcmp(src,"money")){
		sprintf(value[i], "%d", pPlayer->money );
	}
	else if (!strcmp(src, "ping")) {
		sprintf(value[i], "%d", ( pPlayer->lat_samples ? (int)(pPlayer->lat_sum / pPlayer->lat_samples) : 0 ) );
	}
	else if (!strcmp(src, "time")) {
		int time = gpGlobals->time - pPlayer->play_time;
	    sprintf(value[i], "%d:%02d", (int)(time / 60), (int)(time % 60)  );
	}
	else if (!strcmp(src,"wonid")){
		sprintf(value[i], "%u", GETPLAYERWONID(pPlayer->pEdict) );
	}
	else if (!strcmp(src,"authid")){
		sprintf(value[i], "%s", GETPLAYERAUTHID(pPlayer->pEdict) );
	}
	else if (!strcmp(src,"userid")){
		sprintf(value[i], "%d", GETPLAYERUSERID(pPlayer->pEdict) );
	}
	else if (!strcmp(src,"ip")){
		return pPlayer->ip;
	}
	else if (!strcmp(src,"id")){
		sprintf(value[i], "%d", pPlayer->index );
	}
	else if (!strcmp(src,"bot")){
		strcpy(value[i], pPlayer->valid ? "0" : "1" );
	}
	else if (!strncmp(src,"I.",2)){
		return ENTITY_KEYVALUE(pPlayer->pEdict,(char*)(src+2));
	}
	else {
		value[i][0]=0;
	}

	return value[i];
}

const char* GetGameValue(const char* src)
{
	static char value[2][MAX_BUF_LEN];
	static int i = 0; i = i ? 0 : 1;

	if (src[0]=='v'&&src[1]=='.'){
		return GetPlayerValue( score.victim, src + 2 );
	}
	else if (src[0]=='k'&&src[1]=='.'){
		return GetPlayerValue( score.killer, src + 2 );
	}
	else if (src[0]=='p'&&src[1]=='.'){
		return GetPlayerValue( score.player, src + 2 );
	}
	else if (src[0]=='p'&&(src[2]=='.'||src[3]=='.')){ // custom player value {p1.xxx}
		int z = (src[2]=='.')?(src[1]-'0'):( ((src[1]-'0')*10) + (src[2]-'0') );    
		return GetPlayerValue( GET_PLAYER_POINTER_I(z) , src + ((src[2]=='.')?3:4)  );
	}
	else if (!strcmp(src,"sm_mode")){
		sprintf(value[i], "%d", score.sm_mode );
	}
	else if (!strncmp(src,"gm.",3)){
		return GetStatsValue( &score.all_gamestats, src + 3 );
	}
	else if (!strncmp(src,"rd.",3)){
		return GetStatsValue( &score.all_roundstats, src + 3 );
	}
	else if (!strncmp(src,"t1.gm.",6)){
		return GetStatsValue( &score.gamestats[score.teamscore_id[0]] , src + 6 );
	}
	else if (!strncmp(src,"t2.gm.",6)){
		return GetStatsValue( &score.gamestats[score.teamscore_id[1]] , src + 6 );
	}
	else if (!strncmp(src,"t1.rd.",6)){
		return GetStatsValue( &score.roundstats[score.teamscore_id[0]] , src + 6 );
	}
	else if (!strncmp(src,"t2.rd.",6)){
		return GetStatsValue( &score.roundstats[score.teamscore_id[1]] , src + 6 );
	}
	else if (src[0]=='C'&&src[1]=='.'){
		sprintf(value[i], "%s", CVAR_GET_STRING(src + 2) ); // {C.xxx} get cvar value
	}
	else if (src[0]=='%'&&src[1]=='.'){ // {%.xxx} get sm var value
		return getVar(src+2);
	}
	else if (src[0]=='I'&&src[1]=='.'){ // {I.xxx} get localinfo value
		return LOCALINFO((char*)(src+2));
	}
	else if (src[0]=='V'&&src[1]=='.'&&src[2]&&src[3]=='.'){ // {V.1.xxx} means get values from player under this set (this can be index or nick)
		int getv = src[2] - '0';
		if ((!score.best||score.best_type!=getv+12)&&getv<=parsePos){
			int len = parseVault[getv].end - parseVault[getv].begin;
			char me[64]; strncpy(me,&parseMsg[parseVault[getv].begin], len); me[len] = 0;
			if ( parseVault[getv].type == V_STRING )	{
				for(int z = 1; z <= gpGlobals->maxClients; ++z) {
					if (!players[z].ingame || strcmp(STRING(players[z].pEdict->v.netname), me)) continue;
					score.best = GET_PLAYER_POINTER_I(z);
					score.best_type = getv+12;
					break;
				}
			}
			else {
				int getid = atoi(me);
				if (getid > 0 && getid < 33) {
					score.best = GET_PLAYER_POINTER_I(getid);
					score.best_type = getv+12;
				}
			}
		}
		return GetPlayerValue( score.best, src + 3 );
	}
	else if (src[0]=='V'&&src[1]=='.'){ // {V.1} means get first value from enigne message
		int getv = src[2]-'0';
		if (getv > parsePos){
			value[i][0] = 0;
		}
		else {
			int len = parseVault[getv].end - parseVault[getv].begin;
			strncpy(value[i],&parseMsg[parseVault[getv].begin], len);
			value[i][len] = 0;
		}
	}
	else if (src[0]=='W'&&src[1]=='.'){ //covert num to word - {W.2} gives "two" 
		UTIL_IntToString( atoi( src + 2 ) , value[i]);
	}
	else if (src[0]=='R'&&src[1]=='.'){ //rnd - {R.2} gives 0, 1 or 2
		sprintf(value[i], "%d", RANDOM_LONG(0, atoi(src+2) ) );
	}
	else if (!strcmp(src,"maxplayers")){
		sprintf(value[i], "%d", gpGlobals->maxClients );
	}
	else if (!strcmp(src,"currentmap")){
		sprintf(value[i], "%s", STRING(gpGlobals->mapname) );
	}
	else if (!strcmp(src,"timeleft")){
		if (!mp_timelimit->value) return "0";
		int rem = (score.game_timeleft + mp_timelimit->value * 60) - gpGlobals->time;
		sprintf(value[i], "%d:%02d", (rem / 60),(rem % 60) );
	}
	else if (!strcmp(src,"thetime")){
		time_t td;	time(&td);
		strftime(value[i],sizeof(value),"%m/%d/%Y - %H:%M:%S",localtime(&td));
	}
	else if (!strcmp(src,"weekday")){
		time_t td;	time(&td);
		strftime(value[i],sizeof(value[0]),"%A",localtime(&td));
	}
	/*else if (!strcmp(src,"month")){
		time_t td;	time(&td);
		strftime(value[i],sizeof(value[0]),"%B",localtime(&td));
	}
	else if (!strcmp(src,"day")){
		time_t td;	time(&td);
		strftime(value[i],sizeof(value[0]),"%d",localtime(&td));
	}
	else if (!strcmp(src,"year")){
		time_t td;	time(&td);
		strftime(value[i],sizeof(value[0]),"%Y",localtime(&td));
	}*/
	else if (!strcmp(src,"gmtime")){
		sprintf(value[i], "%.2f", gpGlobals->time );
	}
	else if (!strcmp(src,"playersnum")){
		int num = 0;
		for(int w = 1; w <= gpGlobals->maxClients; ++w)
			if (players[w].ingame) ++num;
		sprintf(value[i], "%d", num);
	}
	else
	{
		return src;
	}

	return value[i];
}

const char* CheckCondition(const char* ask, const char* value, const char* yes, const char* no, int operation)
{
	const char* result = NULL;
	bool is_true = false;
	
	if ( isDigit( ask ) )
	{
		float fValue = atof( ask );
		
		switch (operation)
		{
		case '&':
			is_true = ( (int)fValue & ReadFlags( value ) ) ? true : false;
			break;
		case '=':
			is_true = (fValue == atof( value )) ? true : false;
			break;
		case '<':
			is_true = (fValue < atof( value )) ? true : false;
			break;
		case '>':
			is_true = (fValue > atof( value )) ? true : false;
			break;
		case '!':
			is_true = (fValue != atof( value )) ? true : false;
			break;
		default:
			is_true = fValue ? true : false;
		}
	}
	else if (operation)
	{ 
		switch (operation)
		{
		case '&':
			is_true = (strstr(ask, value ))? true : false;
			break;
		case '=':
			is_true = (!strcmpi(ask, value ))? true : false;
			break;
		case '<':
			is_true = (strcmpi(ask, value ) < 0)? true : false;
			break;
		case '>':
			is_true = (strcmpi(ask, value ) > 0)? true : false;
			break;
		case '!':
			is_true = (strcmpi(ask, value )) ? true : false;
			break;
		default:
			is_true = *ask ? true : false;
		}			
	}
	else
	{
		is_true = true;	
	}	
	
	if ( is_true )
	{
		result = ( strcmp( yes,"#skip!" ) ) ? yes : NULL;
	}
	else
	{
		result = ( strcmp( no,"#skip!" ) ) ? no : NULL;
	}

	return result;
}

const char* GetConditionedValue(const char* src, int* length)
{
	static char message[MAX_LEVELS][MAX_DATA_LEN];
	static int level = -1;

	if ( ++level == MAX_LEVELS )
	{
		message[level][ 0 ] = 0;
		return message[level--];
	}

	int len = 0;

	static char szName[MAX_LEVELS][MAX_BUF_LEN]; 
	static char szValue[MAX_LEVELS][MAX_BUF_LEN]; 
	static char szState1[MAX_LEVELS][MAX_BUF_LEN]; 
	static char szState2[MAX_LEVELS][MAX_BUF_LEN];

	szName[level][0] = 0;
	szValue[level][0] = 0;
	szState1[level][0] = 0;
	szState2[level][0] = 0;

	int iName = 0; //this is a name of value to get
	int iValue = 0; //this is an value to compate with requested value
	int iState1 = 0; //text on true
	int iState2 = 0; //text on false

	const char* begin = src;
	int operation = 0;
	bool question = false;

//first see what the expresion is ( maybe { x }  or  { x ? y : z }  )

	while ( *src ) 
	{
		if ( *src == '{' ) // { x ? y : z } - recurence
		{
			int cond_len = 0;

			const char* cond = GetConditionedValue(++src,&cond_len);

			if ( !cond )
			{
				level--;
				return NULL;
			}

			while (*cond) // write from cond_reply to dest
			{
				szName[level][iName++] = *cond++;
			}

			src += cond_len;
			continue;
		}
		else if ( *src == '}' )
		{
			src++;
			break;
		}
		else if ( *src == '&' || *src == '<' || *src == '>' || *src == '=' || *src == '!' || *src == '*' ||
					 *src == '/' ||  *src == '-' ||  *src == '+' )
		{
			operation = *src++;

			while ( *src != '?' && *src != '}' )
			{

				if ( *src == '{' ) // { x ? y : z } - recurence
				{
					int cond_len = 0;
					
					const char* cond = GetConditionedValue(++src,&cond_len);

					if ( !cond )
					{
						level--;
						return NULL;
					}
					
					while (*cond) // write from cond_reply to dest
					{
						szValue[level][iValue++] = *cond++;
					}
					
					src += cond_len;
					continue;
				}

				szValue[level][iValue++] = *src++;
			}

			continue;
		}
		else if ( *src == '?' )
		{
			question = true;
			
			src++;

			while ( *src )
			{
				if ( *src == '{' ) // { x ? y : z } - recurence
				{
					int cond_len = 0;
					
					const char* cond = GetConditionedValue(++src,&cond_len);
					
					if ( !cond )
					{
						level--;
						return NULL;
					}
					
					while (*cond) // write from cond_reply to dest
					{
						szState1[level][iState1++] = *cond++;
					}
					
					src += cond_len;
					continue;
				}
				else if ( *src == ':' && *(src - 1) != '\\' )
				{
					src++; //skip this ':'

					while ( *src )
					{				
						if ( *src == '{' ) // { x ? y : z } - recurence
						{
							int cond_len = 0;
							
							const char* cond = GetConditionedValue(++src,&cond_len);

							if ( !cond )
							{
								level--;
								return NULL;
							}
							
							while (*cond) // write from cond_reply to dest
							{
								szState2[level][iState2++] = *cond++;
							}
							
							src += cond_len;
							continue;
						}

						if ( *src == '}' )
						{
							break;
						}
						
						szState2[level][iState2++] = *src++;	
					}
				}

				if ( *src == '}' )
				{
					break;
				}

				szState1[level][iState1++] = *src++;
			}

			src++;
			break;
		}
		
		szName[level][iName++] = *src++;
	}

	szName[level][iName] = 0;
	szValue[level][iValue] = 0;
	szState1[level][iState1] = 0;
	szState2[level][iState2] = 0;

	const char* value = NULL;
	const char* rootvalue = NULL;

	// this is just an operation on sm var like {%.me+2} or {%.me=0}
	if ( operation && !question) 
	{
		if (szName[level][0] == '%' && szName[level][1] == '.') 
		{
			makeVar( &szName[ level ][2] ,operation, szValue[ level ]);
			message[ level ][ len ] = 0;
			*length = src - begin;
			return message[level--];
		}
		else
		{
			value = doVar(szName[ level ], operation, szValue[ level ]);
			level--;
			*length = src - begin;
			return value;
		}
	}
	else
	{
		rootvalue = GetGameValue(szName[level]);
		value = question ? CheckCondition(rootvalue,szValue[level],szState1[level],szState2[level],operation)	: rootvalue;
	}

	if ( !value )
	{
		level--;
		return NULL;
	}

	while ( *value )
	{
		if ( *value == '\\' )
		{
			switch ( *(value+1) )
			{
			case '\'':
				message[level][len++] = '\"';
				break;
			case 'n':
				message[level][len++] = '\n';
				break;
			default:
				message[level][len++] = *(value+1);
			}
			value+=2;
			continue;
		}
		else if ( *value == '$' && question && rootvalue )
		{
			while (*rootvalue)
			{
				message[level][len++] = *rootvalue++;
			}	

			value++;
			continue;
		}
		else if ( *value == '{' && question ) // { x ? y : z } - recurence
		{
			int cond_len = 0;
			
			const char* cond = GetConditionedValue(++value,&cond_len);

			if ( !cond )
			{
				level--;
				return NULL;
			}
			
			while (*cond) // write from cond_reply to dest
			{
				message[level][len++] = *cond++;
			}
			
			value += cond_len;
			continue;
		}

		message[level][len++] = *value++;
	}

	message[ level ][ len ] = 0;

	*length = src - begin;

	return message[level--];
}

char* UTIL_FillWithVars(const char* src)
{
	static char message[MAX_LEVELS][MAX_DATA_LEN];
	static int level = -1;

	if ( ++level == MAX_LEVELS || !src )
	{
		message[level][0] = 0;
		return message[level--];
	}

	int len = 0;

	while (*src)
	{
		if ( *src == '{' )
		{
			int cond_len = 0;

			const char* cond = GetConditionedValue(++src,&cond_len);

			if (!cond )
			{
				message[level][0] = 0;
				return message[level--];
			}

			while (*cond)
			{
				message[level][len++] = *cond++;
			}

			src += cond_len;
			continue;
		}
		else if ( *src == '\\' )
		{
			switch ( *(src+1) )
			{
			case '\'':
				message[level][len++] = '\"';
				break;
			case 'n':
				message[level][len++] = '\n';
				break;
			default:
				message[level][len++] = *(src+1);
			}
			src+=2;
			continue;
		}

		message[level][len++] = *src++;
	}

	score.best = NULL; //clear special player which you have get from score

	if ( len )
	{
		message[level][len++] = '\n';
	}

	message[level][len] = 0;
	return message[level--];
}

#define MAX_PRINT_MSG 190

void UTIL_ClientPrint( edict_t *pEntity, int msg_dest,  char *fmt, ... )
{
	if ( gmsgTextMsg == 0) {
		LOG_MESSAGE(PLID, "ERROR: (textmsg) no type for message.");
		return;
	}

	va_list			argptr;
	static char		msg_name[512];
	
	va_start ( argptr, fmt );
	vsprintf ( msg_name, fmt, argptr );
	va_end   ( argptr );

	MESSAGE_BEGIN( MSG_ONE, gmsgTextMsg, NULL, pEntity );
	WRITE_BYTE( msg_dest );

	static char message[MAX_PRINT_MSG+2];
	if ( strlen( msg_name ) < MAX_PRINT_MSG ) {
		strcpy( message, msg_name );
	}
	else {
		strncpy( message, msg_name, MAX_PRINT_MSG );
		message[MAX_PRINT_MSG] = 0;		// strncpy doesn't always append the null terminator
	}

	WRITE_STRING( message );
	MESSAGE_END();
}

#define MAX_MENU_CHUNK 176

void UTIL_ShowMenu( player_t *pPlayer, int slots, int displaytime, const char *aMenuList )
{
	if ( gmsgShowMenu == 0) {
		LOG_MESSAGE(PLID, "ERROR: (showmenu) no type for message.");
		return;
	}

	char chunk[MAX_MENU_CHUNK + 1];
	const char *pMenuList = aMenuList;
	int len = 0;

	while ( pMenuList && *pMenuList ) {
		if ( strlen( pMenuList ) < MAX_MENU_CHUNK )	{
			strcpy( chunk, pMenuList );
		}
		else {
			strncpy( chunk, pMenuList, MAX_MENU_CHUNK );
			chunk[MAX_MENU_CHUNK] = 0;
		}

		len += strlen( chunk );//Move text to next chunk
		pMenuList = aMenuList + len;
		MESSAGE_BEGIN( MSG_ONE, gmsgShowMenu, NULL, pPlayer->pEdict );	
			WRITE_SHORT( slots );
			WRITE_CHAR( displaytime );
			WRITE_BYTE(*pMenuList ? TRUE : FALSE); //Need more menu?
			WRITE_STRING(chunk);
		MESSAGE_END();
	}
}

void SettingsMenu(player_t *pPlayer, int state)
{
	if (state < 0){ //we have pressed exit
		pPlayer->menu_type = MENU_IDLE;
		return;
	}
	char textmenu[512]; int list_end, list_begin, pages_num = 0, pos_num = 0, index_key = 0, keys = 0;
	for(action_t*s=sm_menu;s;s=s->next) ++pos_num;
	pages_num = (pos_num / 8) + ((pos_num % 8)?1:0);
	int len =(score.mod_id == MOD_CSTRIKE) ? sprintf(textmenu, "\\yStatsMe Menu \\R%d/%d\n\\w\n", state + 1, pages_num) 
		: sprintf(textmenu, "StatsMe Menu %d/%d\n\n", state + 1, pages_num); //Print header
	pPlayer->menu_state = state; //set position in menu (which page)
	list_begin = (state*8); //get begin of list
	list_end = (pos_num > list_begin + 8) ? (list_begin + 8) : pos_num;
	action_t*a=sm_menu;
	for(int c = 0;c<list_begin;c++)
		a=a->next;
	for (int i=list_begin;i<list_end;++i) {
		if (a->cmd && a->cmd[0] == ' ' && !a->cmd[1]){
			len+=sprintf(&textmenu[len], (score.mod_id == MOD_CSTRIKE)?
				(char*)"\\d    %s\\w":(char*)"    %s", UTIL_FillWithVars(a->name) );
			++index_key;
		}
		else {
			len+=sprintf(&textmenu[len], (score.mod_id == MOD_CSTRIKE)?
				(char*)"\\w%d.  %s":(char*)"%d.  %s", index_key+1, UTIL_FillWithVars(a->name) );
			keys += (1<<index_key++); //Add proper key to menu keys
		}
		a = a->next;
	}
	if (list_end < pos_num){
		len+=sprintf(&textmenu[len],(score.mod_id == MOD_CSTRIKE)?
			(char*)"\n\\w9.  More...":(char*)"\n9.  More...");
		keys += (1<<8);
	}
	strcat(&textmenu[len], (score.mod_id == MOD_CSTRIKE)?((state)?("\n\\w0.  Back"):("\n\\w0.  Exit")):
			((state)?("\n0.  Back"):("\n0.  Exit")));
	keys+=(1<<9);
	UTIL_ShowMenu(pPlayer, keys, -1,textmenu);
	pPlayer->menu_type = SETTINGS_MENU;
	pPlayer->menu_keys = keys;
}

void StatsMenu(player_t *pPlayer, int state)
{
	if (state < 0){ //we have pressed exit
		pPlayer->menu_type = MENU_IDLE;
		return;
	}
	pPlayer->menu_pplnum = 0; //get players list
	for (int z=1;z<=gpGlobals->maxClients;++z){
		if (players[z].ingame){
			pPlayer->menu_list[pPlayer->menu_pplnum++] = &players[z];
		}
	}
	int list_end, list_begin, pages_num = 0, index_key = 0, keys = 0; char textmenu[512];
	pages_num = (pPlayer->menu_pplnum / 8) + ((pPlayer->menu_pplnum % 8)?1:0); //get number of all pages
	int len = (score.mod_id == MOD_CSTRIKE) ? sprintf(textmenu, "\\yStats Menu \\R%d/%d\n\\w\n", state + 1, pages_num) //Print header
		: sprintf(textmenu, "Stats Menu %d/%d\n\n", state + 1, pages_num);
	pPlayer->menu_state = state; //set position in menu (which page)
	list_begin = (state*8); //get begin of list
	//check if this is a last page or we are at the middle
	list_end = (pPlayer->menu_pplnum > list_begin + 8) ? (list_begin + 8) : pPlayer->menu_pplnum;
	for (int i=list_begin;i<list_end;++i) {
		len+=sprintf(&textmenu[len], "%d.  %s\n", index_key+1, STRING(pPlayer->menu_list[i]->pEdict->v.netname));
		keys += (1<<index_key++); //Add proper key to menu keys
	}
	if (list_end < pPlayer->menu_pplnum){
		len+=sprintf(&textmenu[len],"\n9.  More...");
		keys += (1<<8);
	}
	strcat(&textmenu[len], (state)?("\n0.  Back"):("\n0.  Exit"));
	keys+=(1<<9);
	UTIL_ShowMenu(pPlayer, keys, -1,textmenu);
	pPlayer->menu_type = STATS_MENU;
	pPlayer->menu_keys = keys;
}

void ClrMenuCmds(player_t *pPlayer)
{
	for (int i = 0; i < 10; ++i)
	{
		if ( pPlayer->menu_cmds[ i ] )
		{
			delete[] pPlayer->menu_cmds[ i ];

			pPlayer->menu_cmds[ i ] = NULL;
		}
	}
}

void DeleteConfiguration()
{
	for(int i = 0; i < MAX_SM_REG_MSGS ; ++i){
		if (!modMsgFunCalls[i])	continue;
		while (modMsgFunCalls[i]){
			fun_call_t*fc_todel = modMsgFunCalls[i];
			modMsgFunCalls[i] = modMsgFunCalls[i]->next;
			while(fc_todel->cond) {//delete all conditions from fun_call_t
				cond_t*cnd_todel = fc_todel->cond;
				fc_todel->cond = fc_todel->cond->next;
				delete[] cnd_todel->con;
				delete cnd_todel;
			}
			delete[] fc_todel->cmd;
			delete fc_todel;
		}
	}
	
	while(sm_motd) {
		motd_t*motd_todel = sm_motd;
		sm_motd = sm_motd->next;
		delete[] motd_todel->msg;
		delete motd_todel;
	}
	
	while(sm_menu) {
		action_t*menu_todel = sm_menu;
		sm_menu = sm_menu->next;
		delete[] menu_todel->name;
		delete[] menu_todel->cmd;
		delete menu_todel;
	}
	
	while(sm_alias) {
		action_t*alias_todel = sm_alias;
		sm_alias = sm_alias->next;
		delete[] alias_todel->name;
		delete[] alias_todel->cmd;
		delete alias_todel;
	}
				
	while(sm_var) {
		myvar_t*myvar_todel = sm_var;
		sm_var = sm_var->next;
		delete myvar_todel;
	}

	while(sm_task) {
		task_t*task_todel = sm_task;
		sm_task = sm_task->next;
		delete[] task_todel->cmd;
		delete task_todel;
	}

	while (sm_grenades){
		grenades_t*to_del = sm_grenades;
		sm_grenades = sm_grenades->next;
		delete to_del;
	}
}

// used to register throwed grenades
void PutGrenade(edict_t* pGre, float time,int type, player_t* pPlayer)
{
	grenades_t* newone = new grenades_t;
	newone->time = gpGlobals->time + time;
	newone->pPlayer = pPlayer;
	newone->pGrenade = pGre;
	newone->index = type;
	newone->prev = NULL;
	newone->next = sm_grenades;
	if (sm_grenades) sm_grenades->prev = newone;
	sm_grenades = newone;
}

// uset to find out whose grenade is disrupting (also auto removes grenades from list if needed)
grenades_t* FindGrenade(edict_t* enemy)
{
	grenades_t* result = NULL;

	for(grenades_t* p = sm_grenades; p ; ) {
		if ( p->time > gpGlobals->time ) {
			if (p->pGrenade == enemy){
				result = p; //why not just return? - coz we must check for outdated grenades
			}
		}
		else{
			// if the grenade is outdated then delete it from list
			grenades_t* next = p->next;
			if (p->prev)
				p->prev->next = next;
			else  
				sm_grenades = next;
			if (next)
				next->prev = p->prev;
			delete p;
			p = next;
			continue;
		}
		p = p->next;
	}

	return result;
}


void FireShot(player_t* pPlayer, int weapon)
{
	pPlayer->wpnstats[weapon].shots++;
	//pPlayer->specstats[???].shots++;
	pPlayer->roundstats.shots++;
	pPlayer->gamestats.shots++;
	pPlayer->monster.shots++;
	
	score.gamestats[pPlayer->team_index].shots++;
	score.roundstats[pPlayer->team_index].shots++;
	score.all_roundstats.shots++;
	score.all_gamestats.shots++;
}

void FireHit(player_t* pVictim,player_t* pAttacker, int weapon, int damage, int aimed_at)
{	

	pAttacker->wpnstats[weapon].damage += damage;
	pAttacker->wpnstats[weapon].hits++;
	pAttacker->wpnstats[weapon].hitgroup[aimed_at]++;
			
	pAttacker->specstats[pVictim->index].damage += damage;
	pAttacker->specstats[pVictim->index].hits++;
	pAttacker->specstats[pVictim->index].hitgroup[aimed_at]++;
	
	pAttacker->roundstats.damage += damage;
	pAttacker->roundstats.hits++;
	pAttacker->roundstats.hitgroup[aimed_at]++;
	
	pAttacker->gamestats.damage += damage;
	pAttacker->gamestats.hits++;
	pAttacker->gamestats.hitgroup[aimed_at]++;
	
	score.gamestats[pAttacker->team_index].damage += damage;
	score.gamestats[pAttacker->team_index].hits++;
	score.gamestats[pAttacker->team_index].hitgroup[aimed_at]++;
	
	score.all_roundstats.damage += damage;
	score.all_roundstats.hits++;
	score.all_roundstats.hitgroup[aimed_at]++;
	
	score.roundstats[pAttacker->team_index].damage += damage;
	score.roundstats[pAttacker->team_index].hits++;
	score.roundstats[pAttacker->team_index].hitgroup[aimed_at]++;
	
	score.all_gamestats.damage += damage;
	score.all_gamestats.hits++;
	score.all_gamestats.hitgroup[aimed_at]++;
	
	pAttacker->monster.damage += damage;
	pAttacker->monster.hits++;
	pAttacker->monster.hitgroup[aimed_at]++;
}

void FireKill(player_t* pVictim, player_t* pKiller, int weapon, int hs, bool tk)
{
	if ( hs ) { //headshot
		pKiller->gamestats.hs++; // game normal score
		pKiller->roundstats.hs++; // round normal score
		pKiller->wpnstats[weapon].hs++; //save in weapon stats
		pKiller->specstats[pVictim->index].hs++; //in round detailed stats
		
		score.gamestats[pKiller->team_index].hs++; //game team score
		score.roundstats[pKiller->team_index].hs++; //round team score
		score.all_roundstats.hs++;
		score.all_gamestats.hs++;
	}
			
	pKiller->wpnstats[weapon].kills++;
	pKiller->specstats[pVictim->index].kills++;
	pKiller->roundstats.kills++;
	pKiller->gamestats.kills++;
	
	score.gamestats[pKiller->team_index].kills++;
	score.roundstats[pKiller->team_index].kills++;
	
	score.all_roundstats.kills++;
	score.all_gamestats.kills++;
	
	pVictim->wpnstats[weapon].deaths++;
	pVictim->specstats[pVictim->index].deaths++; //round detailed
	pVictim->roundstats.deaths++;
	pVictim->gamestats.deaths++;
	
	score.gamestats[pVictim->team_index].deaths++;
	score.roundstats[pVictim->team_index].deaths++;
	
	score.all_roundstats.deaths++;
	score.all_gamestats.deaths++;
	
	if (tk) {//team killing
		pKiller->wpnstats[weapon].tks++;
		pKiller->specstats[pVictim->index].tks++; //round detailed
		pKiller->roundstats.tks++;
		pKiller->gamestats.tks++;
		score.gamestats[pKiller->team_index].tks++;
		score.roundstats[pKiller->team_index].tks++;
		score.all_roundstats.tks++;
		score.all_gamestats.tks++;
	}
	else {				
		pKiller->monster_time = gpGlobals->time + sm_monsterfreq->value;
		pKiller->monster.kills++;
		if (hs)	pKiller->monster.hs++;
	}
}

char* GetFightList(player_t *pPlayer, bool attackers)
{
	score.player = pPlayer;
	player_t* pLastK = pPlayer->killer;
	player_t* pLastV = pPlayer->victim;
	szBuffor[0] = 0;
	int len = 0;

	for (int a = 1; a <= gpGlobals->maxClients; ++a) 
	{		
		if ( attackers )	
		{
			if ( players[a].ingame && players[a].specstats[pPlayer->index].hits ) 
			{
				pPlayer->killer = &players[a];
				pPlayer->killer->custom_stats = &players[a].specstats[pPlayer->index];

				len += sprintf( &szBuffor[len], UTIL_FillWithVars( sm_attackers->string )  );

				if ( len ) { //exception - remove '\n' from the end of position and convert to "\n"
					szBuffor[len-1] = '\\';
					szBuffor[len] = 'n';
					szBuffor[++len] = 0;
				}
			}
		}
		else
		{
			if ( pPlayer->specstats[a].hits )
			{
				pPlayer->victim = &players[a];
				pPlayer->victim->custom_stats = &players[a].specstats[pPlayer->index];
				pPlayer->custom_stats = &pPlayer->specstats[a];

				len += sprintf( &szBuffor[len], UTIL_FillWithVars( sm_victims->string )  );	
				
				if ( len ) {
					szBuffor[len-1] = '\\';
					szBuffor[len] = 'n';
					szBuffor[++len] = 0;	
				}	
			}	
		}
	}

	pPlayer->killer = pLastK;
	pPlayer->victim = pLastV;

	return szBuffor;
}


// Fill vars with player stats and all weapon stats
char* MakeStats(player_t *pPlayer, char* pList)
{
	score.player = pPlayer;

	int len = sprintf(pList,UTIL_FillWithVars(sm_playerstats->string));

	for (int i = 0; i < MAX_WEAPONS ; ++i)	
	{
		if ( pPlayer->wpnstats[i].shots )
		{ 
			pPlayer->custom_stats = &pPlayer->wpnstats[i];
			len += sprintf(&pList[len],UTIL_FillWithVars(sm_weaponstats->string));
		}
	}

	return pList;
}

bool FloodProtection( player_t *pPlayer, float* time )
{
	if ( !sm_flood->value )
	{
		return true;
	}

	if ( *time < gpGlobals->time )
	{
		*time = gpGlobals->time + sm_flood->value;

		return true;
	}

	return false;
}