
#ifndef STATSME_H
#define STATSME_H

#include <time.h>			// time()
#include <enginecallback.h>		// ALERT()
#include <sdk_util.h>			// UTIL_VarArgs()
#include <api_info.h>

#define VDATE			"10/07/2002"
#define VVERSION		"2.6.4"
#define RC_VERS_DWORD	2,6,4,0	// Version Windows DLL Resources in res_meta.rc
#define VNAME			"STATSME"
#define VAUTHOR			"OLO"
#define VURL			"www.olo.counter-strike.pl"
#define VLOGTAG			"STATSME"
#define VRANKVERSION	5		// Version of save layout for statsme.log 

// Various strings for the Windows DLL Resources in res_plugin.rc
#define RC_COMMENTS		"STATSME"
#define RC_DESC			"STATSME - Metamod Plugin DLL"
#define RC_FILENAME		"statsme_mm.dll"
#define RC_INTERNAL		"STATSME"
#define RC_COPYRIGHT	"Copyright 2002 OLO"

#define ACCURACY_FORMULA(a,b) ((a) ? ( ((float)b * 100.00) / (float)a ) : 0.00 )// a - shots, b - hits
#define EFFEC_FORMULA(a,b) ((b + a) ? ( ((float)a * 100.00) / (float)( b + a ) ) : 0.00 )// a - kills, b - deaths

// parts of body for hits
#define HITGROUP_GENERIC	0 // none
#define HITGROUP_HEAD		1
#define HITGROUP_CHEST		2
#define HITGROUP_STOMACH	3
#define HITGROUP_LEFTARM	4
#define HITGROUP_RIGHTARM	5
#define HITGROUP_LEFTLEG	6
#define HITGROUP_RIGHTLEG	7

// menu ids
enum {
	MENU_IDLE,			//no menu
	STATS_MENU,
	SETTINGS_MENU,
	CUSTOM_MENU			//sm_showmenu
};

// types of data from events (engine msgs)
enum {
	V_FLOAT,
	V_BYTE,
	V_STRING
};

// keeps where value from engine msg begins and ends and what type it is
struct msg_vault_t {
	int begin;
	int end;
	int type; //string, float, byte
};

struct quotes_t {
	const char *sInt;
	const char *sFloat; 
	const char *sStr;
	int	quotes;
};

struct myvar_t {
	char name[32];
	char string[256];
	myvar_t* next;
};

// tasks
struct task_t {
	char* cmd;
	float time;
	float repeat;
	int flags;
	task_t* next;
};

// conditions for sm_register
struct cond_t {
	int	pos;		//position of value to check
	int	len;		//length of value to cmpare
	char* con;		//value to compare
	cond_t* next;
};

// sm_register
struct fun_call_t {
	char*cmd;			// commnad to execute
	bool player;		// receiver is a specified player
	bool world;			// receivers are all players
	bool once;			// allow/prohibit to call multpile times (f.e. this same event send to diffrent players)
	bool done;			// does this func has been already excetued and can't be called again?
	bool fillme;		// does it should be filled with extra info ( { x ? y : z } and so on )?
	int len;			// length of this command
	int where;			// diffrences from 64 (my internal usage)
	int quotes;			// does this call uses quotes
	int	 flag;			// to check with sm_mode, if not set in sm_mode then event is not called
	float stamp;		// save when this function has been called (for 'once' boolean)
	cond_t* cond;		// list of conditions (filter)
	fun_call_t*next;
};

// stats - used by diffrent structures to store some stats - some fields may be not used
struct stat_t {
	const char*	name;	// name of weapon, team or whatever
	int	shots;
	int	hits;
	int	clip;
	int	ammo;
	int	hs;
	int	kills;
	int	tks;
	int	damage;
	int	deaths;
	int	hitgroup[10];	// hits in diff. parts of body
	int	score;
	int switcher;
};

struct player_t {
	edict_t*	pEdict;
	int			index;
	int			play_time;
	bool		valid;					// is this a bot or player
	bool		ingame;
	char*		team_name;
	int			team_index;
	char		ip[32];

	float		monster_time;
	stat_t		monster;				// monster kill stats
	stat_t		gamestats;
	stat_t		roundstats;
	stat_t		specstats[33];			// stats that keeps attacked players (round stats)
	stat_t		wpnstats[MAX_WEAPONS];	// game weapon stats

	stat_t*		custom_stats;			// internal usage
	stat_t*		weapon;					// stats of currently used weapon
	int			aiming_at;				// on what part of body the player is aming - chest, head, stomach etc.
	int			wpnid;					// id of currently used weapon

	player_t*	killer;
	player_t*	victim;

	int			health;
	int			armor;
	int			money;

	int			menu_id;
	int			menu_type;
	int			menu_keys;
	int			menu_state;
	float		menu_time;
	player_t*	menu_list[33];			// list of players for stats menu
	int			menu_pplnum;			// number of players in stats menu
	char*		menu_cmds[10];			// custom server commands for custom menu (set by sm_setmenu)

	int			lat_sum;
	int			lat_samples;
	float		lat_time;
			
	float		say_flood;
	float		alias_flood;
	float		connect_time;			// internal usage for sm_clexec

	player_t*	bleeding;			// for DOD
	int			bleeding_weapon;
	int			bleeding_aiming;

	int			damage_deathmsg;
	int			damage_damage;
	int			damage_aim;
	int			damage_weapon;
	player_t*	damage_attacker;
};

struct grenades_t {
	player_t*	pPlayer;
	edict_t*	pGrenade;
	float		time;
	int			index;

	grenades_t*next;
	grenades_t*prev;
};

void DeleteConfiguration();

class statsme_t {
public:

	statsme_t() { };
	~statsme_t() { DeleteConfiguration(); };

	int			mod_id;
	char		mod_name[32];
	char		debug_name[128];

	int			sm_mode;

	float		loadcfg;

	float		c4_timing;			// delay between counting down
	int			c4_timer;


	char		teamscore_name[4][32];
	int			teamscore_id[4];

	char		team_name[4][32];	// unassigned, spectator, ct (aliases), t (axis)
	int			team_num;

	player_t*	killer;
	player_t*	victim;
	player_t*	player;

	player_t*	best;				// keeps best result to not recalculate thesame player multiple times during filling with score
	int			best_type;			// in case we want best result but from other activity

	stat_t		gamestats[4];		// stats for specified teams
	stat_t		roundstats[4];
	stat_t		all_gamestats;		// for all players
	stat_t		all_roundstats;

	float		game_timeleft;
	float		game_restarting;
	float		endround_time;
	float		endmap_time;
	float		newround_time;
	float		task_time;

	time_t		last_reset;			// of top15 rank
};

// filled during weaponlist engine message
struct wpn_sel_t {
	char name[24];			// name of the weapon when selecting it
	short int iId;			// the weapon id value
	short int ammo_slot;	//slot for ammo
};

// server rank which are displayed by top15 cmd
struct rank_t {
	char name[36];
	int points;		// points by which the rank is sort
	int	hits;		// hits taken from all weapons
	int	shots;		// shots taken from all weapons
	int	kills;		// frags
	int	deaths;		// his deaths
	int	dmg;		// overall damage done by this player (this is not taken health)
	int	tks;		// teamkilling
	int	hs;			// headshots
	float eff;		// efficiency (kills to deaths)
	float acc;		// accuracy (hits to shots)
};

struct fun_t
{
	void (*function)(void*);
};

extern fun_t modMsgs[MAX_REG_MSGS];
extern fun_t modMsgEnds[MAX_REG_MSGS ];

#define GETPLAYERAUTHID	(*g_engfuncs.pfnGetPlayerAuthId)
#define CLIENT_PRINTF (*g_engfuncs.pfnClientPrintf)
#define GET_PLAYER_POINTER(e) &players[ENTINDEX(e)]
#define GET_PLAYER_POINTER_I(i) &players[i]

extern int	gmsgDeathMsg;
extern int	gmsgDamage;
extern int	gmsgCurWeapon;
extern int	gmsgTextMsg;
extern int	gmsgTeamInfo;
extern int	gmsgWeaponList;
extern int	gmsgResetHUD;
extern int	gmsgMOTD;
extern int	gmsgServerName;
extern int	gmsgintermission;
extern int	gmsgHealth;
extern int	gmsgBattery;
extern int  gmsgShowMenu;
extern int  gmsgSendAudio;
extern int  gmsgTeamScore;
extern int	gmsgMoney;
extern int	gmsgAmmoX;
extern int	gmsgBleeding;
extern int	gmsgBandaging;

#define MAX_SM_REG_MSGS		288 // MAX_REG_MSGS is 256

#define SM_DEATHMSG			256 //SM_DeathMsg
#define SM_ROUNDEND			257 //SM_RoundEnd
#define SM_ROUNDSTART		258 //SM_RoundStart
#define SM_MONSTERKILL		259 //SM_MonsterKill
#define SM_C4TIMER			260 //SM_C4Timer
#define SM_DAMAGE			261 //SM_Damage
#define SM_INFOCHANGED		262 //SM_InfoChanged
#define SM_CONNECT			263 //SM_Connect
#define SM_DISCONNECT		264 //SM_Disconnect
#define SM_PUTINSERVER		265 //SM_PutInServer

extern char	szBuffor[2048];
extern char	parseMsg[768];
extern int parsePos;

extern void (*endfunction)(void*);
extern fun_call_t* modMsgFunCalls[ MAX_SM_REG_MSGS ];
extern fun_call_t* modMsgCalledCvar;
extern msg_vault_t parseVault[10];
extern player_t* mPlayer;
extern int mState;

#define	MOD_UNKNOW	0
#define	MOD_CSTRIKE	1
#define	MOD_DOD		2
#define	MOD_TFC		3 // not supported yet

// keeps motd for players
struct motd_t {
	char*msg;
	motd_t*next;
};

// menu and aliases
struct action_t {
	char*name;
	char*cmd;
	int flags;
	action_t*next;
};

extern player_t		players[33];
extern rank_t		rank[65];
extern wpn_sel_t	weapon_select[MAX_WEAPONS];
extern myvar_t*		sm_var;
extern task_t*		sm_task;
extern action_t*	sm_alias;
extern action_t*	sm_menu;
extern grenades_t*	sm_grenades;
extern motd_t*		sm_motd;
extern statsme_t	score;
extern int			player_bits;

extern cvar_t	*statsme_version;
extern cvar_t	*sm_refresh;
extern cvar_t	*sm_logformat;
extern cvar_t	*sm_monsternum;
extern cvar_t	*sm_playerstats;
extern cvar_t	*sm_weaponstats;
extern cvar_t	*sm_menupassword;
extern cvar_t	*sm_monsterfreq;
extern cvar_t	*sm_rankcmd;
extern cvar_t	*sm_statsmecmd;
extern cvar_t	*sm_statscmd;
extern cvar_t	*sm_killsforrank;
extern cvar_t	*sm_resetrank;
extern cvar_t	*sm_latlogformat;
extern cvar_t	*sm_latlogfreq;
extern cvar_t	*sm_ranklogfile;
extern cvar_t	*sm_timelogformat;
extern cvar_t	*sm_logformat2;
extern cvar_t	*sm_attackers;
extern cvar_t	*sm_victims;
extern cvar_t	*sm_resetonrestart;
extern cvar_t	*mp_timelimit;
extern cvar_t	*sm_logmenuaction;
extern cvar_t	*sm_switchinfo;
extern cvar_t	*sm_cfgfile;
extern cvar_t	*sm_flood;
extern cvar_t	*sm_debug;

#define FL_SCORE	(1<<0)	//intermission stats

bool FloodProtection( player_t *pPlayer, float* time );
bool CallsMe(player_t* pPlayer, const char* arg);
int ReadFlags(const char* c);
const char* GET_MOD_NAME(int* id);
char* UTIL_SplitTextMessage(char *pMessage);
char* UTIL_FillWithVars(const char* src);
char* MakeStats(player_t *pPlayer, char* pList);
char* GetFightList(player_t *pPlayer, bool attackers);
void UTIL_ShowMOTD( edict_t *client , char *pMotd, const char *szName);
void UTIL_HudMessage(const hudtextparms_t &textparms, char *pMessage, edict_t *pEntity);
void UTIL_ClientPrint( edict_t *pEntity, int msg_dest, char *fmt, ... );
void UTIL_IntToString(int iValue, char *szOutput);
void SettingsMenu(player_t *pPlayer, int state);
void StatsMenu(player_t *pPlayer, int state);
void LoadRank(void);
void SaveRank(void);
void statsme_init(void);
void PlayerConnect(edict_t *pEntity, const char *pszName, const char *pszAddress);
void PlayerPutInServer( player_t* pPlayer );
void PlayerDisconnect( player_t *pPlayer);
void AddPlayerToRank(player_t* pPlayer);
void ShowTopScore(player_t* pPlayer);
void ClrMenuCmds(player_t *pPlayer);
void UTIL_ShowMenu( player_t *pPlayer, int slots, int displaytime, const char *aMenuList );
void CmdTell(void);
void CmdAlias(void);
void CmdClExec(void);
void CmdMotd(void);
void CmdShowMenu(void);
void CmdMenuCmds(void);

// CS support
void Client_DeathMsg_CS(void*);
void Client_Damage_CS(void*);
void Client_CurWeapon_CS(void*);
void Client_TextMsg_CS(void*);
void Client_TeamInfo_CS(void*);
void Client_ResetHUD_End_CS(void*);
void Client_SendAudio_CS(void*);
void Client_TeamScore_CS(void*);
void Client_Money_CS(void*);
void Client_SM_DeathMsg_CS(void*);
void Client_SM_Damage_CS(void*);
void Client_WeaponList_CS(void*);
void Client_Battery_CS(void*);
void Client_ShowMenu_CS(void*);
void Client_intermission_End_CS(void*);
void Client_AmmoX_CS(void*);
void Client_Health_CS(void*);
void EmitSound_CS(edict_t *entity, int channel, const char *sample, float volume, float attenuation, int fFlags, int pitch);
void SetModel_CS(edict_t *e, const char *m);
void StartFrame_CS( void );

// DOD support
void Client_Health_DOD(void*);
void Client_Bleeding_DOD(void*);
void Client_Bandaging_DOD(void*);
void Client_DeathMsg_DOD(void*);
void SetModel_DOD(edict_t *e, const char *m);
void EmitSound_DOD(edict_t *entity, int channel, const char *sample, /*int*/float volume, float attenuation, int fFlags, int pitch);
void StartFrame_DOD( void );


void StartFrame( void );
void FireShot(player_t* pPlayer, int weapon);
void FireHit(player_t* pVictim,player_t* pAttacker ,int weapon, int damage, int aimed_at);
void FireKill(player_t* pVictim, player_t* pKiller, int weapon, int hs, bool tk);

grenades_t* FindGrenade(edict_t* enemy);
void PutGrenade(edict_t* pGre, float time, int type, player_t* pPlayer);

void PARSE_MESSAGE_BEGIN(int msg_type, player_t* target = NULL, int index = 0);
void PARSE_MESSAGE_END(void);
void PARSE_WRITE_BYTE(int iValue);
void PARSE_WRITE_CHAR(int iValue);
void PARSE_WRITE_SHORT(int iValue);
void PARSE_WRITE_LONG(int iValue);
void PARSE_WRITE_ANGLE(float flValue);
void PARSE_WRITE_COORD(float flValue);
void PARSE_WRITE_STRING(const char *sz);
void PARSE_WRITE_ENTITY(int iValue);

#endif // STATSME_H
