
#include <extdll.h>			// always
#include <sdk_util.h>		// REG_SVR_COMMAND, etc
#include <meta_api.h>		// Plugin_info, etc
#include <time.h>
#include "statsme.h"	// me

rank_t rank[65]; //rank of players

inline void swap(rank_t& a, rank_t& b) 
{
	rank_t c=a; a=b; b=c;
}

void sortrank(rank_t *table,int left,int right) 
{
	if (left < right) {
		int m=left;
		for(int i=left+1;i<=right;i++){
			if (table[i].points>table[left].points)
				swap(table[++m],table[i]);
		}
		swap(table[left],table[m]);
		sortrank(table,left,m-1);
		sortrank(table,m+1,right);
	}
}

// used to set points for rank - set here your own rules for scoring the player
void givepoints(rank_t*pos)
{
	pos->points = pos->eff + pos->acc + ( pos->hs / 5 );

	if ( pos->kills < sm_killsforrank->value )
		pos->points /= 2;
}

void AddPlayerToRank(player_t* pPlayer)
{
	const char* nick = STRING(pPlayer->pEdict->v.netname);

	int a = 0;
	
	while ( strncmp(nick,rank[a].name,32) && a < 64 ) ++a; //find index of name
	
	if (a == 64) { //doesn't exists - so clear last slot and add him there
		memset( &rank[a] , 0 , sizeof( rank_t ) );
		strncpy( rank[a].name, nick , 32 );
	}

	rank[a].tks += pPlayer->gamestats.tks;
	rank[a].dmg += pPlayer->gamestats.damage;
	rank[a].deaths += pPlayer->gamestats.deaths;
	rank[a].kills += pPlayer->gamestats.kills;
	rank[a].shots += pPlayer->gamestats.shots;
	rank[a].hits += pPlayer->gamestats.hits;
	rank[a].hs += pPlayer->gamestats.hs;
	rank[a].acc = ACCURACY_FORMULA(rank[a].shots,rank[a].hits);
	rank[a].eff = EFFEC_FORMULA(rank[a].kills,rank[a].deaths);
	givepoints(&rank[a]);
	sortrank(rank,0,64);
}

void ShowTopScore(player_t* pPlayer)
{
	static char *spaces = "..........................................."; //used to format rank list

	int len = sprintf(szBuffor,"#  nick                     kills/deaths   hits/shots/hs.   eff.%%   acc.%%\n");

	for (int i = 0; i < 15 && rank[i].name[0] ; ++i) 
	{
		len += sprintf(&szBuffor[len], UTIL_VarArgs("%d. %s%s%d/%d   %d/%d/%d   %.2f   %.2f\n",
			i + 1, rank[i].name, spaces+(int)(strlen(rank[i].name)*1.6), rank[i].kills , 
			rank[i].deaths,rank[i].hits, rank[i].shots,rank[i].hs,  rank[i].eff, rank[i].acc));

	}

	UTIL_ShowMOTD(pPlayer->pEdict,szBuffor, "Top 15");
}

void LoadRank()
{
#ifndef __linux__
	FILE *bfp = fopen(UTIL_VarArgs("%s\\%s",score.mod_name,sm_ranklogfile->string), "rb");
#else
	FILE *bfp = fopen(UTIL_VarArgs("%s/%s",score.mod_name,sm_ranklogfile->string), "rb");
#endif
	if (!bfp) return; //file doesn't exist
	int version = 0;
	fread(&version, 1 , sizeof(int) , bfp);
	if (version == VRANKVERSION){ //proceed only if version of log is the same
		time_t when, now;
		time(&now); //get current time
		fread(&when, 1 , sizeof(time_t) , bfp); //read when stats were last cleared
		if ( sm_resetrank->value && difftime(now, when ) > (sm_resetrank->value * 3600) ) {
			memset(&rank, 0 ,sizeof(rank)); //reset stats
			time(&score.last_reset); //save date of this reset
		} else {
			fread(&rank, 1,sizeof(rank), bfp);
		}
	}
	fclose(bfp);
}

void SaveRank()
{
#ifndef __linux__
	FILE *bfp = fopen(UTIL_VarArgs("%s\\%s",score.mod_name,sm_ranklogfile->string), "wb");
#else
	FILE *bfp = fopen(UTIL_VarArgs("%s/%s",score.mod_name,sm_ranklogfile->string), "wb");
#endif
	if (!bfp) return;  //file doesn't exist
	int version = VRANKVERSION;
	fwrite(&version, 1, sizeof(int) , bfp);
	fwrite(&score.last_reset, 1, sizeof(time_t) , bfp);
	fwrite(&rank, 1, sizeof(rank) , bfp);
	fclose(bfp);
}